#
# $Header: /home/cvs/tcllian/src/tcl/contacts.tcl,v 1.2 2006/04/03 12:36:50 dossy Exp $
# $Id: contacts.tcl,v 1.2 2006/04/03 12:36:50 dossy Exp $
#


namespace eval ::contacts {
    variable script [info script]
    variable contacts {}
    variable numContacts 0

    proc start {} {
        variable entry_id
        # Add the Tkcon entry to the group.
        if {![info exists entry_id]} {
            set entry_id -1
        }
        if {$entry_id == -1} {
            set entry_id [trillian listAddEntry \
                    section_id $::tcllian::section_id \
                    parent_id $::tcllian::group_id \
                    real_name "contacts" text "Contacts" \
                    callback [list ::contacts::listCallback]]
        }

        variable event_type
        if {![info exists event_type]} {
            set event_type "Tcllian: Contacts: Show window"
        }
        variable event_id
        if {![info exists event_id]} {
            set event_id -1
        }
        if {$event_id == -1} {
            set event_id [trillian eventsActionRegister $event_type $event_type \
                    [list ::contacts::listCallback list_leftDoubleClick]]
        }
    }

    proc stop {} {
        variable entry_id
        trillian listRemoveEntry $entry_id
        set entry_id -1

        variable event_type
        variable event_id
        trillian eventsActionUnregister $event_id $event_type
        set event_id -1
    }

    proc listCallback {event args} {
        switch -- $event {
            list_leftDoubleClick {
                makeWindow
            }
        }
        return 0
    }

    proc makeWindow {} {
        if {![winfo exists .contacts]} {
            toplevel .contacts
            entry .contacts.input
            frame .contacts.output
            listbox .contacts.output.contactList \
                -yscrollcommand {.contacts.output.sb set} \
                -font courier -width 40
            scrollbar .contacts.output.sb -command {.contacts.output.contactList yview}
            pack .contacts.input  -side top -fill x -expand false
            pack .contacts.output -side bottom -fill both -expand true
            pack .contacts.output.contactList -side left -fill both -expand true
            pack .contacts.output.sb -side right -fill y

            bind .contacts.input <KeyRelease> [list ::contacts::searchContacts .contacts]
            bind .contacts.output.contactList <Double-1> [list ::contacts::openMessageWindow .contacts.output.contactList 0]
            bind .contacts.input <Return> [list ::contacts::openMessageWindow .contacts.output.contactList 1]
            bind .contacts.input <Escape> [list destroy .contacts]

            wm protocol .contacts WM_DELETE_WINDOW {destroy .contacts}
        }

        update
        wm deiconify .contacts
        focus .contacts.input
    }

    proc openMessageWindow {win {destroy 0}} {
        variable contacts

        set item [lindex $contacts [$win curselection]]
        if {![llength $item]} {
            tcllian messagebox "Please select a contact and try again."
            return
        }
        #puts "openMessageWindow: $item"
        if {$destroy} {
            destroy [winfo toplevel $win]
        }
        foreach {medium id . name} $item break
        trillian messageSend $medium $id $name ""
    }

    proc searchContacts {win} {
        variable contacts
        variable numContacts

        array set connections {}
        foreach row [trillian connectionEnumerate all] {
            foreach {medium id name status section} $row break
            set connections([list $medium $id $section]) $name
        }
        set text [$win.input get]
        set contacts [contacts "*${text}*"]
        $win.output.contactList delete 0 end
        set size 0
        foreach row $contacts {
            foreach {medium id target name status section . .} $row break
            set self $connections([list $medium $id $section])
            $win.output.contactList insert end [format "%-10s %-30s (%s: %s)" $status $target $medium $self]
            incr size
        }
        $win.output.contactList selection set 0
        wm title $win "Contacts ($size of $numContacts)"
    }

    proc contacts {{pattern *}} {
        variable numContacts
        set contactList [allContacts]
        set numContacts [llength $contactList]
        array set contactArray [splitContacts $contactList]

        set dispNames [string tolower $contactArray(displayName)]
        set realNames [string tolower $contactArray(realName)]
        set pattern [string tolower $pattern]

        set dispIndices [lsearch -glob -all $dispNames $pattern]
        set realIndices [lsearch -glob -all $dispNames $pattern]

        set allIndices [lsort -unique -integer [concat $dispIndices $realIndices]]

        set results {}
        foreach elem $allIndices {
            lappend results [lindex $contactList $elem]
        }

        return $results
    }

    proc allContacts {} {
        set results {}
        foreach connection [trillian connectionEnumerate all] {
            foreach {medium id name status section} $connection break
            if {[string length $section]} {
                set results [concat $results \
                        [trillian contactlistEnumerate $medium $id $section]]
            } else {
                set results [concat $results \
                        [trillian contactlistEnumerate $medium $id]]
            }
        }
        return [lsort $results]
    }

    proc resource {} {
        variable script

        puts "Sourcing: $script"
        source $script
    }

    proc splitContacts {contactList} {
        array set results {
            displayName {}
            realName {}
        }
        
        foreach contact $contactList {
            lappend results(displayName) [lindex $contact 2]
            lappend results(realName) [lindex $contact 3]
        }

        return [array get results]
    }
}

return
