#
# tcllian::load livejournal.tcl
#
# lj::account <username> <password>
# lj::start
#

package require http
package require md5
package require tdom

namespace eval ::lj {
    variable config

    proc start {} {
        variable config
        if {![info exists config(url)]} { set config(url) http://www.livejournal.com }
        if {![info exists config(section_id)]} { set config(section_id) -1 }
        if {![info exists config(group_id)]} { set config(group_id) -1 }
        if {![info exists config(name)]} { set config(name) {} }
        if {![info exists config(password)]} { set config(password) {} }
        if {![info exists config(after)]} { set config(after) {} }
        if {![info exists config(frequency)]} { set config(frequency) 5 }
        if {![info exists config(limit)]} { set config(limit) 0 }
        if {![info exists config(cookies)]} { set config(cookies) [getCookies] }
        refresh
    }

    proc stop {} {
        variable config
        variable friends
        catch {after cancel $config(after)}
        trillian listRemoveEntry -section $config(section_id)
        set config(section_id) -1
        catch {unset friends}
    }

    proc account {username password} {
        variable config
        set config(name) $username
        set config(password) $password
        return
    }

    proc config {args} {
        variable config
        foreach {key value} $args {
            set config($key) $value
        }
        return
    }

    proc getCookies {} {
        variable config

        set cookie_ljuniq {}
        set cookie_ljsession {}

        set url [format "%s/login.bml" $config(url)]
        set t [http::geturl $url]
        upvar #0 $t state
        set body $state(body)
        set meta $state(meta)
        http::cleanup $t

        foreach {key value} $meta {
            if {$key eq "Set-Cookie" && [string match "ljuniq=*" $value]} {
                regexp {^(.*?); .*$} $value -> cookie_ljuniq
            }
        }

        regexp {<input type='hidden' name='chal' id='login_chal' value='(.*?)' />} $body -> chal
        set md5pwd [string tolower [md5::md5 -hex $config(password)]]
        set resp [string tolower [md5::md5 -hex "$chal$md5pwd"]]

        set url [format "%s/login.bml" $config(url)]
        set query [http::formatQuery \
            chal $chal \
            response $resp \
            user $config(name) \
            password "" \
            action:login "Log in..." \
            expire never \
            bindip no]
        set t [http::geturl $url -query $query \
            -headers [list Cookie $cookie_ljuniq]]
        upvar #0 $t state
        set body $state(body)
        set meta $state(meta)
        http::cleanup $t

        foreach {key value} $meta {
            if {$key eq "Set-Cookie" && [string match "ljsession=*" $value]} {
                regexp {^(.*?); .*$} $value -> cookie_ljsession
            }
        }

        return [list $cookie_ljuniq $cookie_ljsession]
    }
    
    proc refresh {} {
        variable config

        if {$config(section_id) == -1} {
            set config(section_id) [trillian listAddEntry \
                section 1 previous_id -2 \
                real_name "lj" text "LiveJournal ($config(name))"]
        }
        if {$config(group_id) == -1} {
            set config(group_id) [trillian listAddEntry \
                group 1 section_id $config(section_id) \
                previous_id -1 expanded 1 \
                real_name "Your LiveJournal" text "Your LiveJournal"]
            set link [format "%s/users/%s" $config(url) $config(name)]
            trillian listAddEntry \
                section_id $config(section_id) parent_id $config(group_id) \
                real_name "view" text "View your journal" \
                callback [list ::lj::listEntryCallback [list trillian browserOpen $link]]
            set link [format "%s/update.bml" $config(url)]
            trillian listAddEntry \
                section_id $config(section_id) parent_id $config(group_id) \
                real_name "update" text "Update your journal" \
                callback [list ::lj::listEntryCallback [list trillian browserOpen $link]]
            set link [format "%s/users/%s/friends" $config(url) $config(name)]
            trillian listAddEntry \
                section_id $config(section_id) parent_id $config(group_id) \
                real_name "friends" text "View your Friends page" \
                callback [list ::lj::listEntryCallback [list trillian browserOpen $link]]
            trillian listAddEntry \
                section_id $config(section_id) parent_id $config(group_id) \
                real_name "configWatchlist" text "Configure your watchlist" \
                callback [list ::lj::listEntryCallback [list ::lj::configWatchlistDialog]]
        }
        return ;# for now

        set url [format "%s/users/%s/data/foaf" $config(url) $config(name)]
        set t [http::geturl $url -headers [list Cookie [join $config(cookies) "; "]]]
        httpFoafCallback $t

        if {[lsearch [after info] $config(after)] < 0} {
            set config(after) [after [expr {$config(frequency) * 60 * 1000}] \
                [list ::lj::refresh]]
        }
    }
    
    proc listEntryCallback {action event args} {
        switch -- $event {
            list_leftDoubleClick {
                eval $action
            }
        }
    }
    
    proc configWatchlistDialog {} {
        set w .ljwatchlist
        if {[winfo exists $w]} {
            focus $w
            return
        } else {
            toplevel $w
            pack [text $w.t -width 60 -height 20]
            pack [frame $w.f] -side bottom
            pack [button $w.f.save -text "Save" -command [list ::lj::button %W $w.t]] -side left
            pack [button $w.f.cancel -text "Cancel" -command [list destroy $w]]
        }
    }

    proc httpFoafCallback {token} {
        variable config
        variable friends

        set xml [http::data $token]
        http::cleanup $token

        #trillian systraySetAlert . "::lj::httpFoafCallback $token"
        if {[catch {dom parse $xml dom} err]} {
            trillian systraySetAlert . "LJ: FOAF parse error: $err"
            return
        }
        set result [catch {
            $dom documentElement doc
            foreach node [$doc selectNodes {/rdf:RDF/foaf:Person/foaf:knows}] {
                set nick [$node selectNodes {string(foaf:Person/foaf:nick)}]
                if {![info exists friends($nick)]} {
                    set group_id [trillian listAddEntry \
                        group 1 section_id $config(section_id) \
                        previous_id -1 real_name $nick text $nick]
                    set friends($nick) $group_id
                } else {
                    set group_id $friends($nick)
                    trillian listClearEntry $group_id
                }

                set url [format "%s/users/%s/data/rss" $config(url) $nick]
                set t [http::geturl $url -headers [list Cookie [join $config(cookies) "; "]]]
                httpRssCallback $nick $t
            }
        } err]
        $dom delete
        if {$result} {
            trillian systraySetAlert . "LJ: FOAF parse error: $err"
        }
    }

    proc httpRssCallback {nick token} {
        variable config
        variable friends

        set xml [http::data $token]
        http::cleanup $token

        #trillian systraySetAlert . "::lj::httpRssCallback $nick $token"
        if {[catch {dom parse $xml dom} err]} {
            trillian systraySetAlert . "LJ: RSS parse error: $err"
            return
        }
        set result [catch {
            set group_id $friends($nick)
            $dom documentElement doc
            set items 0
            foreach node [$doc selectNodes {/rss/channel/item}] {
                set title [$node selectNodes {string(title)}]
                set link [$node selectNodes {string(link)}]
                set description [$node selectNodes {string(description)}]
                if {![string length $title]} {
                    set title "(no subject)"
                }
                trillian listAddEntry \
                    section_id $config(section_id) parent_id $group_id \
                    real_name $title text $title tooltip $description \
                    callback [list trillian browserOpen $link]
                incr items
                if {$config(limit) && $items == $config(limit)} {
                    break
                }
            }
            trillian listUpdateText $group_id "$nick ($items)" $nick
        } err]
        $dom delete
        if {$result} {
            trillian systraySetAlert . "LJ: RSS parse error: $err"
        }
    }
}

